# DO NOT EDIT THIS FILE!
#
# This file is generated from the CDP specification. If you need to make
# changes, edit the generator and regenerate all of the modules.

from . import (
    accessibility,
    animation,
    audits,
    autofill,
    background_service,
    bluetooth_emulation,
    browser,
    css,
    cache_storage,
    cast,
    console,
    dom,
    dom_debugger,
    dom_snapshot,
    dom_storage,
    debugger,
    device_access,
    device_orientation,
    emulation,
    event_breakpoints,
    extensions,
    fed_cm,
    fetch,
    file_system,
    headless_experimental,
    heap_profiler,
    io,
    indexed_db,
    input_,
    inspector,
    layer_tree,
    log,
    media,
    memory,
    network,
    overlay,
    pwa,
    page,
    performance,
    performance_timeline,
    preload,
    profiler,
    runtime,
    schema,
    security,
    service_worker,
    storage,
    system_info,
    target,
    tethering,
    tracing,
    web_audio,
    web_authn,
)
